# encoding: utf-8

class ImageUploader < CarrierWave::Uploader::Base

  # Include RMagick or MiniMagick support:
  # include CarrierWave::RMagick
  include CarrierWave::MiniMagick

  # Choose what kind of storage to use for this uploader:
  # storage Rails.env.production? ? :fog : :file
  # storage :file

  # Override the directory where uploaded files will be stored.
  # This is a sensible default for uploaders that are meant to be mounted:
  # def store_dir
  #  "uploads/#{Rails.env}/#{model.class.to_s.underscore}/#{model.id}"
  # end
  
  def cache_dir
    Rails.root.join 'tmp/uploads'
  end

  process resize_to_limit: [2048, 2048]
  process :optimize
  # process :crop

  # version :large do
  #   #process :crop
  #   #process :resize_to_fit => [60, 60]
  #   #process :resize_to_fill => [60, 60]
  #   #process :resize_to_limit => [250, 250]
  #   process :resize_to_fit => [250, 250]
  # end

  # version :medium do
  #   process :resize_to_fit => [120, 120]
  # end

  # version :small do
  #   process :resize_to_fit => [60, 60]
  # end

  # version :tiny do
  #   process :resize_to_fit => [25, 25]
  # end

  def crop
    if model.crop_x.present?
      resize_to_limit(540, 540)
      manipulate! do |img|
        x = model.crop_x.to_i
        y = model.crop_y.to_i
        w = model.crop_w.to_i
        h = model.crop_h.to_i
        img.crop!(x, y, w, h)
      end
    end
  end

  def optimize
    manipulate! do |img|
      img.combine_options do |c|
        c.strip
        c.quality '85'
        c.depth '8'
        c.interlace 'Line'
      end
      img
    end
  end

  # Provide a default URL as a default if there hasn't been a file uploaded:
  # def default_url
  #   # For Rails 3.1+ asset pipeline compatibility:
  #   # ActionController::Base.helpers.asset_path("fallback/" + [version_name, "default.png"].compact.join('_'))
  #
  #   "/images/fallback/" + [version_name, "default.png"].compact.join('_')
  # end

  # Process files as they are uploaded:
  # process :scale => [200, 300]
  #
  # def scale(width, height)
  #   # do something
  # end

  # Create different versions of your uploaded files:
  # version :thumb do
  #   process :scale => [50, 50]
  # end

  # Add a white list of extensions which are allowed to be uploaded.
  # For images you might use something like this:
  def extension_whitelist
    %w(jpg jpeg png)
  end

  # Override the filename of the uploaded files:
  # Avoid using model.id or version_name here, see uploader/store.rb for details.
  # def filename
  #   "something.jpg" if original_filename
  # end

end
